<?php

require_once(dirname(__FILE__).'/permissionsException.php');

class FilesHelper {

    public static function remove_dir($dir, $self = true, $excluded = array()) {
        try
        {
           $it = new RecursiveDirectoryIterator($dir, RecursiveDirectoryIterator::SKIP_DOTS);
            $files = new RecursiveIteratorIterator($it,
                         RecursiveIteratorIterator::CHILD_FIRST);
            foreach($files as $file) {
                if (count($excluded) > 0) {
                    $parts = preg_split('/\\\\|\//', $file->getRealPath());
                    if (count(array_intersect($excluded, $parts)) > 0) {
                        continue;
                    }
                }
                if ($file->isDir()) {
                    FilesHelper::delete_folder($file->getRealPath());
                } else {
                    FilesHelper::delete_file($file->getRealPath());
                }
            }
            if ($self) FilesHelper::delete_folder($dir);
        }
        catch (Exception $e)
        {
            throw new PermissionsException('Unable to remove the directory: ' . $dir);
        }
    }

    public static function delete_folder($path) {
        @rmdir($path);
        if (is_dir($path) && file_exists($path)) {
            throw new PermissionsException('Unable to remove the folder: ' . $path);
        }
    }

    public static function delete_file($file) {
        if (is_file($file))
            @unlink($file);
        if (file_exists($file)) {
            throw new PermissionsException('Unable to remove the file: ' . $file);
        }
    }

    public static function generate_fso($path, $recursive = true) {
        if (is_file($path)) {
            $content = FilesHelper::read_file($path);

            $type = 'text';
            $ext = pathinfo($path, PATHINFO_EXTENSION);

            if (in_array($ext, array('jpg', 'jpeg', 'bmp', 'png', 'gif', 'svg'))) {
                $type = 'data';
                $content = base64_encode($content);
            }

            return array(
                'type' => $type,
                'content' => $content
            );
        }

        if (!is_dir($path))
            return array();

        $result = array(
            'type' => 'dir',
            'items' => array()
        );

        if ($handle = opendir($path)) {
            while (($name = readdir($handle)) !== false) {
                if (preg_match('#^\.#', $name))
                    continue;

                $result['items'][$name] = FilesHelper::generate_fso($path . "/" . $name, $recursive);
            }
            closedir($handle);
        }
        return $result;
    }

    /**
     * @param $path
     * @param $recursive
     */
    public static function enumerate_files($path, $recursive = true) {
        $files = array();
        if (!is_dir($path)) {
            return $files;
        }

        if ($handle = opendir($path)) {
            while (($name = readdir($handle)) !== false) {
                if (preg_match('#^\.#', $name)) {
                    continue;
                }

                if (is_dir($path . "/" . $name) && $recursive) {
                    $files = array_merge($files, FilesHelper::enumerate_files($path . "/" . $name));
                } else {
                    $files[] = array('path' => $path . '/' . $name);
                }
            }
            closedir($handle);
        }

        return $files;
    }

    /**
     * @param $source
     * @param $destination
     */
    public static function copy_recursive($source, $destination, $change_file = null, $replace_data = null) {
        if(is_file($source)) {
            if($change_file || $replace_data) {
                $ext = pathinfo($source, PATHINFO_EXTENSION);
                $text_file = in_array($ext, array('php', 'tpl', 'info', 'inf'));

                $content = $change_file && $text_file ? call_user_func($change_file, $source) : FilesHelper::read_file($source);
                if (isset($replace_data) && $text_file) {
                    $data = call_user_func_array($replace_data['method'], array($content, $destination, $replace_data['old_name'], $replace_data['new_name']));
                    FilesHelper::write_file($data['destination'], $data['content']);
                }
                else {
                    FilesHelper::write_file($destination, $content);
                }
            } else {
                FilesHelper::copy_file($source, $destination);
            }
        } elseif(is_dir($source)) {
            if(!is_dir($destination)) {
                if(!mkdir($destination)) {
                    return;
                }
            }
            if ($dh = opendir($source)) {
                while (($file = readdir($dh)) !== false) {
                    if('.' == $file || '..' == $file) {
                        continue;
                    }
                    FilesHelper::copy_recursive($source . '/' . $file, $destination . '/' . $file, $change_file, $replace_data);
                }
                closedir($dh);
            }
        }
    }

    public static function create_dir($path)
    {
        if (!file_exists($path)) {
            @mkdir($path, 0777, true);
            if (!is_dir($path) && !file_exists($path)) {
                throw new PermissionsException('Unable to create the directory: ' . $path);
            }
        }
    }

    public static function write_file($path, $content, $flags = 0)
    {
        set_error_handler('permissionErrorHandler');
        $message = 'Unable to create or write to the file: ';
        $dir = dirname($path);
        FilesHelper::create_dir($dir);
        $chmodResult = true;
        if (file_exists($path)) {
            $chmodResult = chmod($path, 0755);
        }
        $return = file_put_contents($path, $content, $flags);
        if (false === $return || !$chmodResult) {
            throw new PermissionsException($message . $path);
        }
        restore_error_handler();
        chmod($path, 0755);
    }

    public static function read_file($path)
    {
        set_error_handler('permissionErrorHandler');
        $message = 'Unable to read the file: ';
        $e = new PermissionsException($message . $path);
        if (!file_exists($path) || !is_readable($path))
            throw $e;
        $content = file_get_contents($path);
        if (false === $content)
            throw $e;

        restore_error_handler();
        return $content;
    }

    public static function rename_file($oldname, $newname)
    {
        if (!rename($oldname, $newname)) {
            throw new PermissionsException('Unable to rename the file: ' . $oldname);
        }
    }

    public static function copy_file($source, $dest)
    {
        $dir = dirname($dest);
        FilesHelper::create_dir($dir);
        if (!file_exists($source) || !copy($source, $dest)) {
            throw new PermissionsException('Unable to copy the file: ' . $source . ' to ' .  $dest);
        }
    }

    public static function empty_dir($dir, $hard = false) {
        if(!is_dir($dir)) {
            return;
        }
        $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($dir,
                FilesystemIterator::KEY_AS_PATHNAME | FilesystemIterator::CURRENT_AS_FILEINFO | FilesystemIterator::SKIP_DOTS),
            RecursiveIteratorIterator::CHILD_FIRST);

        foreach ($iterator as $path) {
            if ($path->isDir()) {
                FilesHelper::delete_folder($path->__toString());
            } else {
                FilesHelper::delete_file($path->__toString());
            }
        }
        if($hard) {
            FilesHelper::delete_folder($dir);
        }
    }
    
    public static function normalize_path($path) {
        $root = ($path[0] === '/') ? '/' : '';
        $segments = preg_split('/[\\/\\\\]/', trim($path, '/'));
        $ret = array();
        foreach ($segments as $segment) {
            if (($segment == '.') || empty($segment)) {
                continue;
            }
            if ($segment == '..') {
                array_pop($ret);
            } else {
                array_push($ret, $segment);
            }
        }
        return $root . implode('/', $ret);
    }
    
    public static function replace_theme_name($content, $destination, $old_name, $new_name) {
        $dest = (strpos($destination, $old_name . INF_EXT) !== false)  // .info will be find too
                ? str_replace($old_name . INF_EXT, $new_name . INF_EXT, $destination)
                : $destination;
        $result = self::replace_content($content, $old_name, $new_name);
        return array('destination' => $dest, 'content' => $result);
    }
    
    public static function replace_content($content, $old, $new) {
        $result = (strpos($content, $old) !== false)
                ? str_replace($old, $new, $content)
                : $content;
        return $result;
    }

    public static function check_dir_permissions($path)
    {
        return is_dir($path) && is_writable($path) && is_readable($path);
    }

    public static function check_file_permissions($path)
    {
        return file_exists($path) && is_writable($path) && is_readable($path);
    }
    
    public static function check_files($path, $recursive = true, $check = false, $rootOnly = false, $names_only = false) {
        $files = array();
        if (!is_dir($path)) {
            return $files;
        }

        if ($check && ((is_dir($path) && !FilesHelper::check_dir_permissions($path)) || !FilesHelper::check_file_permissions($path)))
            throw new PermissionsException('Please check permissions for: ' . $path);

        if ($rootOnly)
            return $files;

        if ($handle = opendir($path)) {
            while (($name = readdir($handle)) !== false) {
                if (preg_match('#^\.#', $name)) {
                    continue;
                }

                if (is_dir($path . "/" . $name) && $recursive) {
                    if ($check && !FilesHelper::check_dir_permissions($path))
                        throw new PermissionsException('Please check permissions for: ' . $path);

                    $files = array_merge($files, FilesHelper::check_files($path . "/" . $name, $recursive, $check, $rootOnly, $names_only));
                } else {
                    if ($check && !FilesHelper::check_file_permissions($path))
                        throw new PermissionsException('Please check permissions for: ' . $path);

                    if ($names_only)
                        $files[] = $name;
                    else
                        $files[] = array('path' => $path . '/' . $name, 'name' => $name);
                }
            }
            closedir($handle);
        }

        return $files;
    }
}

function permissionErrorHandler($errno, $errstr, $errfile, $errline, $errcontext)
{
    $msg = (isset($errcontext['message']) ? $errcontext['message'] : '') . (isset($errcontext['path']) ? $errcontext['path'] : '');
    throw new PermissionsException($msg);
}